@@ -4,6 +4,8 @@ import logging
 
 from flask import Flask
 
+from alerta.version import __version__
+
 LOG_FORMAT = '%(asctime)s - %(name)s[%(process)d]: %(levelname)s - %(message)s'
 
 app = Flask(__name__)
@@ -23,12 +25,15 @@ else:
     app.logger.addHandler(stderr_handler)
     app.logger.setLevel(logging.INFO)
 
+app.logger.info('Starting alerta version %s ...', __version__)
+
 from alerta.app.database import Mongo
 db = Mongo()
 
 import views
 import webhooks.views
 import management.views
+import auth
 
 
 def main():
