@@ -10,11 +10,10 @@ from alerta.app.switch import Switch, SwitchState
 from alerta.app.utils import crossdomain
 from alerta.app.metrics import Gauge, Counter, Timer
 from alerta import build
+from alerta.version import __version__
 
 LOG = logging.getLogger(__name__)
 
-import pkg_resources  # part of setuptools
-__version__ = pkg_resources.require("alerta")[0].version
 
 switches = [
     Switch('auto-refresh-allow', 'Allow consoles to auto-refresh alerts', SwitchState.ON),
