@@ -10,10 +10,17 @@ from flask import make_response, request, current_app
 from functools import update_wrapper
 
 from alerta.app import app, db
-from alerta.alert import Alert
+from alerta.app.metrics import Timer
+from alerta.plugins import load_plugins, RejectException
 
 LOG = app.logger
 
+plugins = load_plugins()
+
+duplicate_timer = Timer('alerts', 'duplicate', 'Duplicate alerts', 'Total time to process number of duplicate alerts')
+correlate_timer = Timer('alerts', 'correlate', 'Correlated alerts', 'Total time to process number of correlated alerts')
+create_timer = Timer('alerts', 'create', 'Newly created alerts', 'Total time to process number of new alerts')
+
 
 class DateEncoder(json.JSONEncoder):
     def default(self, obj):
@@ -45,94 +52,6 @@ def jsonp(func):
     return decorated
 
 
-def authenticate():
-    return jsonify(status="error", message="authentication required"), 401
-
-
-
-def verify_token(token):
-    if db.is_token_valid(token):
-        return True
-
-    url = 'https://www.googleapis.com/oauth2/v1/tokeninfo?access_token=' + token
-    response = requests.get(url)
-    token_info = response.json()
-
-    if 'error' in token_info:
-        LOG.warning('Token authentication failed: %s', token_info['error'])
-        return False
-
-    if 'audience' in token_info:
-        if token_info['audience'] != app.config['OAUTH2_CLIENT_ID']:
-            LOG.warning('Token supplied was for different web application')
-            return False
-
-    if 'email' in token_info:
-        if not ('*' in app.config['ALLOWED_EMAIL_DOMAINS']
-                or token_info['email'].split('@')[1] in app.config['ALLOWED_EMAIL_DOMAINS']
-                or db.is_user_valid(token_info['email'])):
-            LOG.info('User %s not authorized to access API', token_info['email'])
-            return False
-    else:
-        LOG.warning('No email address present in token info')
-        return False
-
-    db.save_token(token)
-    return True
-
-
-def verify_api_key(key):
-    if not db.is_key_valid(key):
-        return False
-    db.update_key(key)
-    return True
-
-
-def get_user_info(token):
-    url = 'https://www.googleapis.com/oauth2/v1/userinfo?access_token=' + token
-    response = requests.get(url)
-    user_info = response.json()
-
-    if 'error' in user_info:
-        LOG.warning('Token authentication failed: %s', user_info['error'])
-        return None
-
-    return user_info
-
-
-def auth_required(func):
-    @wraps(func)
-    def decorated(*args, **kwargs):
-
-        if not app.config['AUTH_REQUIRED']:
-            return func(*args, **kwargs)
-
-        if 'Authorization' in request.headers:
-            auth = request.headers['Authorization']
-            if auth.startswith('Token'):
-                token = auth.replace('Token ', '')
-                if not verify_token(token):
-                    return authenticate()
-            elif auth.startswith('Key'):
-                key = auth.replace('Key ', '')
-                if not verify_api_key(key):
-                    return authenticate()
-            else:
-                return authenticate()
-        elif 'token' in request.args:
-            token = request.args['token']
-            if not verify_token(token):
-                return authenticate()
-        elif 'api-key' in request.args:
-            key = request.args['api-key']
-            if not verify_api_key(key):
-                return authenticate()
-        else:
-            return authenticate()
-        return func(*args, **kwargs)
-    return decorated
-
-
 PARAMS_EXCLUDE = [
     '_',
     'callback',
@@ -279,6 +198,7 @@ def crossdomain(origin=None, methods=None, headers=None,
 
             h['Access-Control-Allow-Origin'] = origin
             h['Access-Control-Allow-Methods'] = get_methods()
+            h['Access-Control-Allow-Credentials'] = 'true'
             h['Access-Control-Max-Age'] = str(max_age)
             if headers is not None:
                 h['Access-Control-Allow-Headers'] = headers
@@ -289,54 +209,38 @@ def crossdomain(origin=None, methods=None, headers=None,
     return decorator
 
 
-def parse_notification(notification):
+def process_alert(incomingAlert):
 
-    notification = json.loads(notification)
+    for plugin in plugins:
+        try:
+            incomingAlert = plugin.pre_receive(incomingAlert)
+        except RejectException:
+            raise
+        except Exception as e:
+            raise RuntimeError('Error while running pre-receive plug-in: %s', e)
+        if not incomingAlert:
+            raise SyntaxError('Plug-in pre-receive hook did not return modified alert')
 
-    if notification['Type'] == 'SubscriptionConfirmation':
+    try:
+        if db.is_duplicate(incomingAlert):
+            started = duplicate_timer.start_timer()
+            alert = db.save_duplicate(incomingAlert)
+            duplicate_timer.stop_timer(started)
+        elif db.is_correlated(incomingAlert):
+            started = correlate_timer.start_timer()
+            alert = db.save_correlated(incomingAlert)
+            correlate_timer.stop_timer(started)
+        else:
+            started = create_timer.start_timer()
+            alert = db.create_alert(incomingAlert)
+            create_timer.stop_timer(started)
+    except Exception as e:
+        raise RuntimeError(e)
 
-        return Alert(
-            resource=notification['TopicArn'],
-            event=notification['Type'],
-            environment='Production',
-            severity='informational',
-            service=['Unknown'],
-            group='CloudWatch',
-            text='%s <a href="%s" target="_blank">SubscribeURL</a>' % (notification['Message'], notification['SubscribeURL']),
-            origin='AWS/CloudWatch',
-            event_type='cloudwatchAlarm',
-            create_time=datetime.datetime.strptime(notification['Timestamp'], '%Y-%m-%dT%H:%M:%S.%fZ'),
-            raw_data=notification,
-        )
+    for plugin in plugins:
+        try:
+            plugin.post_receive(alert)
+        except Exception as e:
+            raise RuntimeError('Error while running post-receive plug-in: %s', e)
 
-    elif notification['Type'] == 'Notification':
-
-        alarm = json.loads(notification['Message'])
-
-        return Alert(
-            resource='%s:%s' % (alarm['Trigger']['Dimensions'][0]['name'], alarm['Trigger']['Dimensions'][0]['value']),
-            event=alarm['Trigger']['MetricName'],
-            environment='Production',
-            severity=cw_state_to_severity(alarm['NewStateValue']),
-            service=[alarm['AWSAccountId']],
-            group='CloudWatch',
-            value=alarm['NewStateReason'],
-            text=notification['Subject'],
-            attributes=alarm['Trigger'],
-            origin=alarm['Trigger']['Namespace'],
-            event_type='cloudwatchAlarm',
-            create_time=datetime.datetime.strptime(notification['Timestamp'], '%Y-%m-%dT%H:%M:%S.%fZ'),
-            raw_data=alarm
-        )
-
-
-def cw_state_to_severity(state):
-
-    if state == 'ALARM':
-        return 'major'
-    elif state == 'INSUFFICIENT_DATA':
-        return 'warning'
-    elif state == 'OK':
-        return 'normal'
-    else:
-        return 'unknown'
+    return alert
