@@ -1,30 +1,23 @@
 import datetime
 
 from collections import defaultdict
-from flask import request, render_template
+from flask import g, request, render_template
+from uuid import uuid4
 
 from alerta.app import app, db
 from alerta.app.switch import Switch
-from alerta.app.utils import jsonify, jsonp, auth_required, parse_fields, crossdomain
+from alerta.app.auth import auth_required
+from alerta.app.utils import jsonify, jsonp, parse_fields, crossdomain, process_alert
 from alerta.app.metrics import Timer
 from alerta.alert import Alert
 from alerta.heartbeat import Heartbeat
-from alerta.plugins import load_plugins, RejectException
+from alerta.plugins import RejectException
 
 LOG = app.logger
 
-@app.before_first_request
-def setup():
-    global plugins
-    plugins = load_plugins()
-
-
 # Set-up metrics
 gets_timer = Timer('alerts', 'queries', 'Alert queries', 'Total time to process number of alert queries')
 receive_timer = Timer('alerts', 'received', 'Received alerts', 'Total time to process number of received alerts')
-duplicate_timer = Timer('alerts', 'duplicate', 'Duplicate alerts', 'Total time to process number of duplicate alerts')
-correlate_timer = Timer('alerts', 'correlate', 'Correlated alerts', 'Total time to process number of correlated alerts')
-create_timer = Timer('alerts', 'create', 'Newly created alerts', 'Total time to process number of new alerts')
 delete_timer = Timer('alerts', 'deleted', 'Deleted alerts', 'Total time to process number of deleted alerts')
 status_timer = Timer('alerts', 'status', 'Alert status change', 'Total time and number of alerts with status changed')
 tag_timer = Timer('alerts', 'tagged', 'Tagging alerts', 'Total time to tag number of alerts')
@@ -124,7 +117,7 @@ def get_alerts():
             statusCounts=status_count,
             lastTime=query_time,
             autoRefresh=Switch.get('auto-refresh-allow').is_on()
-        ), 404
+        )
 
 @app.route('/alerts/history', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -157,16 +150,13 @@ def get_history():
             message="not found",
             history=[],
             lastTIme=query_time
-        ), 404
+        )
 
 @app.route('/alert', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
 @auth_required
 @jsonp
 def receive_alert():
-    #
-    # A received alert can result in a duplicate, correlated or new alert
-    #
 
     recv_started = receive_timer.start_timer()
     try:
@@ -175,65 +165,23 @@ def receive_alert():
         receive_timer.stop_timer(recv_started)
         return jsonify(status="error", message=str(e)), 400
 
-    if incomingAlert:
-        for plugin in plugins:
-            try:
-                incomingAlert = plugin.pre_receive(incomingAlert)
-            except RejectException as e:
-                return jsonify(status="error", message=str(e)), 403
-            except Exception as e:
-                LOG.warning('Error while running pre-receive plug-in: %s', e)
-            if not incomingAlert:
-                LOG.error('Plug-in pre-receive hook did not return modified alert')
-
     try:
-        if db.is_duplicate(incomingAlert):
-
-            started = duplicate_timer.start_timer()
-            alert = db.save_duplicate(incomingAlert)
-            duplicate_timer.stop_timer(started)
-
-            for plugin in plugins:
-                try:
-                    plugin.post_receive(alert)
-                except Exception as e:
-                    LOG.warning('Error while running post-receive plug-in: %s', e)
-
-        elif db.is_correlated(incomingAlert):
-
-            started = correlate_timer.start_timer()
-            alert = db.save_correlated(incomingAlert)
-            correlate_timer.stop_timer(started)
-
-            for plugin in plugins:
-                try:
-                    plugin.post_receive(alert)
-                except Exception as e:
-                    LOG.warning('Error while running post-receive plug-in: %s', e)
-
-        else:
-            started = create_timer.start_timer()
-            alert = db.create_alert(incomingAlert)
-            create_timer.stop_timer(started)
-
-            for plugin in plugins:
-                try:
-                    plugin.post_receive(alert)
-                except Exception as e:
-                    LOG.warning('Error while running post-receive plug-in: %s', e)
-
+        alert = process_alert(incomingAlert)
+    except RejectException as e:
+        receive_timer.stop_timer(recv_started)
+        return jsonify(status="error", message=str(e)), 403
+    except Exception as e:
         receive_timer.stop_timer(recv_started)
-
-    except Exception, e:
         return jsonify(status="error", message=str(e)), 500
 
+    receive_timer.stop_timer(recv_started)
+
     if alert:
         body = alert.get_body()
         body['href'] = "%s/%s" % (request.base_url, alert.id)
         return jsonify(status="ok", id=alert.id, alert=body), 201, {'Location': '%s/%s' % (request.base_url, alert.id)}
     else:
-        return jsonify(status="error", message="alert insert or update failed"), 500
-
+        return jsonify(status="error", message="insert or update of received alert failed"), 500
 
 @app.route('/alert/<id>', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -251,7 +199,7 @@ def get_alert(id):
         body['href'] = request.base_url
         return jsonify(status="ok", total=1, alert=body)
     else:
-        return jsonify(status="ok", message="not found", total=0, alert=None), 404
+        return jsonify(status="error", message="not found", total=0, alert=None), 404
 
 
 @app.route('/alert/<id>/status', methods=['OPTIONS', 'POST'])
@@ -273,12 +221,6 @@ def set_status(id):
         return jsonify(status="error", message="must supply 'status' as parameter"), 400
 
     if alert:
-        for plugin in plugins:
-            try:
-                plugin.post_receive(alert)
-            except Exception as e:
-                LOG.warning('Error while running post-receive plug-in: %s', e)
-
         status_timer.stop_timer(status_started)
         return jsonify(status="ok")
     else:
@@ -393,7 +335,7 @@ def get_counts():
             total=0,
             severityCounts=severity_count,
             statusCounts=status_count
-        ), 404
+        )
 
 @app.route('/alerts/top10', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -427,7 +369,7 @@ def get_top10():
             message="not found",
             total=0,
             top10=[],
-        ), 404
+        )
 
 @app.route('/environments', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -457,7 +399,7 @@ def get_environments():
             message="not found",
             total=0,
             environments=[],
-        ), 404
+        )
 
 
 @app.route('/services', methods=['OPTIONS', 'GET'])
@@ -488,7 +430,7 @@ def get_services():
             message="not found",
             total=0,
             services=[],
-        ), 404
+        )
 
 
 # Return a list of heartbeats
@@ -523,7 +465,7 @@ def get_heartbeats():
             total=0,
             heartbeats=hb_list,
             time=datetime.datetime.utcnow()
-        ), 404
+        )
 
 @app.route('/heartbeat', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -561,7 +503,7 @@ def get_heartbeat(id):
         body['href'] = request.base_url
         return jsonify(status="ok", total=1, heartbeat=body)
     else:
-        return jsonify(status="ok", message="not found", total=0, heartbeat=None), 404
+        return jsonify(status="error", message="not found", total=0, heartbeat=None), 404
 
 @app.route('/heartbeat/<id>', methods=['OPTIONS', 'DELETE', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -607,7 +549,7 @@ def get_users():
             users=[],
             domains=app.config['ALLOWED_EMAIL_DOMAINS'],
             time=datetime.datetime.utcnow()
-        ), 404
+        )
 
 @app.route('/user', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -615,21 +557,23 @@ def get_users():
 @jsonp
 def create_user():
 
-    if request.json and 'user' in request.json:
-        user = request.json["user"]
-        sponsor = request.json["sponsor"]
-        data = {
-            "user": user,
-            "sponsor": sponsor
-        }
+    if request.json and 'name' in request.json:
+        name = request.json["name"]
+        email = request.json["email"]
+        provider = request.json["provider"]
         try:
-            db.save_user(data)
+            user = db.save_user(str(uuid4()), name, email, provider)
         except Exception as e:
             return jsonify(status="error", message=str(e)), 500
     else:
-        return jsonify(status="error", message="must supply 'user' and 'sponsor' as parameters"), 400
+        return jsonify(status="error", message="must supply user 'name', 'email' and 'provider' as parameters"), 400
+
+    if user:
+        return jsonify(status="ok", user=user), 201, {'Location': '%s/%s' % (request.base_url, user)}
+    else:
+        return jsonify(status="error", message="User with email address already exists"), 409
+
 
-    return jsonify(status="ok"), 201, {'Location': '%s/%s' % (request.base_url, user)}
 
 @app.route('/user/<user>', methods=['OPTIONS', 'DELETE', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -673,7 +617,7 @@ def get_keys():
             total=0,
             keys=[],
             time=datetime.datetime.utcnow()
-        ), 404
+        )
 
 @app.route('/keys/<user>', methods=['OPTIONS', 'GET'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -700,7 +644,7 @@ def get_user_keys(user):
             total=0,
             keys=[],
             time=datetime.datetime.utcnow()
-        ), 404
+        )
 
 @app.route('/key', methods=['OPTIONS', 'POST'])
 @crossdomain(origin='*', headers=['Origin', 'X-Requested-With', 'Content-Type', 'Accept', 'Authorization'])
@@ -709,18 +653,16 @@ def get_user_keys(user):
 def create_key():
 
     if request.json and 'user' in request.json:
-        user = request.json["user"]
-        data = {
-            "user": user,
-            "text": request.json.get("text", "API Key for %s" % user)
-        }
-        try:
-            key = db.create_key(data)
-        except Exception as e:
-            return jsonify(status="error", message=str(e)), 500
+        user = request.json['user']
     else:
         return jsonify(status="error", message="must supply 'user' as parameter"), 400
 
+    text = request.json.get("text", "API Key for %s" % user)
+    try:
+        key = db.create_key(user, text)
+    except Exception as e:
+        return jsonify(status="error", message=str(e)), 500
+
     return jsonify(status="ok", key=key), 201, {'Location': '%s/%s' % (request.base_url, key)}
 
 @app.route('/key/<path:key>', methods=['OPTIONS', 'DELETE', 'POST'])
