@@ -11,7 +11,7 @@ class LogStashOutput(PluginBase):
 
     def pre_receive(self, alert):
 
-        pass
+        return alert
 
     def post_receive(self, alert):
 
@@ -28,3 +28,6 @@ class LogStashOutput(PluginBase):
             raise RuntimeError("logstash exception")
 
         self.sock.close()
+
+        return
+
